@@ -39,7 +39,7 @@ bool FractionField::initialize_frac(const PolyRingFlat *R)
       R->getCoefficients()
           ->cast_to_FractionField()  // disallowed in x-relem.cpp
       ||
-      R->getMonoid()->getNonTermOrderVariables()->len >
+      R->getMonoid()->numNonTermOrderVariables() >
           0)  // disallowed in x-relem.cpp
     use_gcd_simplify = false;
   else
@@ -115,7 +115,7 @@ void FractionField::simplify(frac_elem *f) const
       x = f->numer;
       const RingElement *a = RingElement::make_raw(R_, x);
       const RingElement *b = RingElement::make_raw(R_, y);
-      const RingElement *c = rawGCDRingElement(a, b, NULL, false);
+      const RingElement *c = rawGCDRingElement(a, b, nullptr, false);
 
 #if 0
       // Debugging code
@@ -203,9 +203,9 @@ void FractionField::lower_content(ring_elem &c, const ring_elem g) const
   const RingElement *g1 = RingElement::make_raw(R_, gf->numer);
   const RingElement *g2 = RingElement::make_raw(R_, gf->denom);
 
-  c1 = rawGCDRingElement(c1, g1, NULL, false);
+  c1 = rawGCDRingElement(c1, g1, nullptr, false);
 
-  const RingElement *cc2 = rawGCDRingElement(c2, g2, NULL, false);
+  const RingElement *cc2 = rawGCDRingElement(c2, g2, nullptr, false);
   const RingElement *cc3 = (*c2) * (*g2);
   const RingElement *cc4 = (*cc3) / (*cc2);
 
@@ -614,7 +614,6 @@ ring_elem FractionField::eval(const RingMap *map,
   const Ring *S = map->get_ring();
   const frac_elem *f = FRAC_VAL(a);
   ring_elem top = R_->eval(map, f->numer, first_var);
-  if (S->is_zero(top)) return top;
   ring_elem bottom = R_->eval(map, f->denom, first_var);
   if (S->is_zero(bottom))
     {
@@ -638,16 +637,6 @@ bool FractionField::is_homogeneous(const ring_elem a) const
   return true;
 }
 
-void FractionField::degree(const ring_elem a, monomial d) const
-{
-  const frac_elem *f = FRAC_VAL(a);
-  R_->degree(f->numer, d);
-  monomial e = degree_monoid()->make_one();
-  R_->degree(f->denom, e);
-  degree_monoid()->divide(d, e, d);
-  degree_monoid()->remove(e);
-}
-
 bool FractionField::multi_degree(const ring_elem a, monomial d) const
 {
   const frac_elem *f = FRAC_VAL(a);
@@ -660,7 +649,7 @@ bool FractionField::multi_degree(const ring_elem a, monomial d) const
 }
 
 void FractionField::degree_weights(const ring_elem,
-                                   M2_arrayint,
+                                   const std::vector<int> &,
                                    int &lo,
                                    int &hi) const
 {
@@ -672,7 +661,7 @@ void FractionField::degree_weights(const ring_elem,
 ring_elem FractionField::homogenize(const ring_elem a,
                                     int v,
                                     int deg,
-                                    M2_arrayint wts) const
+                                    const std::vector<int> &wts) const
 {
   int d1, d2, lo1, lo2;
   ring_elem top, bottom;
@@ -697,7 +686,7 @@ ring_elem FractionField::homogenize(const ring_elem a,
 
 ring_elem FractionField::homogenize(const ring_elem a,
                                     int v,
-                                    M2_arrayint wts) const
+                                    const std::vector<int> &wts) const
 {
   const frac_elem *f = FRAC_VAL(a);
   ring_elem top = R_->homogenize(f->numer, v, wts);
