@@ -1,5 +1,5 @@
 ;;; M2.el --- Major mode for editing Macaulay2 source core -*- lexical-binding: t -*-
-;; Version: 1.1
+;; Version: 1.23
 ;; Keywords: languages Macaulay2
 ;; URL: https://github.com/Macaulay2/M2-emacs
 ;; Package-Requires: ((emacs "24.1"))
@@ -50,7 +50,7 @@
   "Major mode for interacting with a Macaulay2 process.\n\n\\{M2-comint-mode-map}"
   (M2-common)
   (setq comint-prompt-regexp M2-comint-prompt-regexp)
-  (add-hook 'comint-output-filter-functions 'M2-info-help nil t)
+  (add-hook 'comint-preoutput-filter-functions 'M2-info-help nil t)
   (setq-local compilation-error-regexp-alist M2-error-regexp-alist)
   (setq-local compilation-transform-file-match-alist
 	      M2-transform-file-match-alist)
@@ -77,11 +77,11 @@
 ;; key bindings
 
 (define-key M2-mode-map "\177" 'backward-delete-char-untabify)
-(define-key M2-mode-map "\^M" 'M2-newline-and-indent)
 ;; (define-key M2-mode-map "}" 'M2-electric-right-brace)
 (define-key M2-mode-map ";" 'M2-electric-semi)
 ;; (define-key M2-mode-map "\^Cd" 'M2-find-documentation)
 (define-key M2-mode-map (kbd "<C-return>") 'M2-send-to-program)
+(define-key M2-mode-map (kbd "<f11>") 'M2-send-to-program)
 (define-key M2-mode-map (kbd "C-c C-j") 'M2-send-line-to-program)
 (define-key M2-mode-map (kbd "C-c C-r") 'M2-send-region-to-program)
 (define-key M2-mode-map (kbd "C-c C-b") 'M2-send-buffer-to-program)
@@ -106,14 +106,14 @@
 (define-key M2-comint-mode-map [ (control C) r ] 'scroll-right)
 ;(define-key M2-comint-mode-map [ f8 ] 'switch-to-completions)
 (define-key M2-comint-mode-map [ (control C) c ] 'switch-to-completions)
-(define-key M2-comint-mode-map "\r" 'M2-send-input)
 ;; (define-key M2-comint-mode-map [ (control C) d ] 'M2-find-documentation)
+(define-key M2-comint-mode-map (kbd "<f11>")
+	    #'M2-send-input-or-get-input-from-demo-buffer)
 
 (mapc
  (function
   (lambda (mode-map)
     (define-key mode-map [ f12 ] 'M2) ; the user may want to make this one global
-    (define-key mode-map [ f11 ] 'M2-send-to-program) ; the user may want to make this one global
     (define-key mode-map [ (meta f12) ] 'M2-demo)
     (define-key mode-map [ (control f11) ] 'M2-switch-to-demo-buffer)
     (define-key mode-map [ (meta f11) ] 'M2-set-demo-buffer)
@@ -148,7 +148,6 @@
      ["Send paragraph to Macaulay2"   M2-send-paragraph-to-program]
      ["Highlight evaluated region"    M2-toggle-blink-region-flag
       :style toggle :selected M2-blink-region-flag]
-     ["Newline and indent"            M2-newline-and-indent]
      ["Electric semicolon"            M2-electric-semi]
      ["Electric right brace"          M2-electric-right-brace]
      ["Electric tab"                  M2-electric-tab]
@@ -159,7 +158,10 @@
   "Menu for Macaulay2 Interaction major mode"
   (append
    '("Macaulay2 Interaction"
-     ["Send to Macaulay2"   M2-send-input]
+     ["Send to Macaulay2"   comint-send-input]
+     ["Get demo input"      M2-get-input-from-demo-buffer]
+     ["Send to M2 or get demo input"
+      M2-send-input-or-get-input-from-demo-buffer]
      ["Go to end of prompt" M2-to-end-of-prompt]
      ["Center point"        M2-position-point]
      ["Jog left"            M2-jog-left]
@@ -351,8 +353,8 @@ can be executed with \\[M2-send-to-program]."
     ;; error messages, e.g.,
     ;; i1 : load "packages/Macaulay2Doc/demo1.m2"; g 2
     ;; packages/Macaulay2Doc/demo1.m2:8:12:(3):[2]: error: division by zero
-    ;;  (1           1)   (2      2)   (3      3)
-    ("\\([^:\n\r\s]+\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
+    ;;  (1                                           1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
      1 2 3)
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; net(FilePosition) (debugging.m2) ;;
@@ -360,14 +362,14 @@ can be executed with \\[M2-send-to-program]."
     ;; start & end line/column numbers, e.g.,:
     ;; i1 : locate (rank, Matrix)
     ;; o1 = m2/matrix1.m2:663:19-666:20
-    ;;  (1              1)   (2      2)   (3      3)   (4      4)   (5      5)
-    ("\\([^\\[:\n\r\s]+\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
+    ;;  (1                                           1)   (2      2)   (3      3)   (4      4)   (5      5)
+    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
      1 (2 . 4) (3 . 5) 0)
     ;; no end line/column numbers, e.g.,:
     ;; i2 : locate makeDocumentTag rank
     ;; o2 = ../Macaulay2Doc/functions/rank-doc.m2:34:0
-    ;;  (1           1)   (2      2)   (3      3)
-    ("\\([^:\n\r\s]+\\):\\([0-9]+\\):\\([0-9]+\\)"
+    ;;  (1                                           1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[^\\[:\n\r\s]+\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)"
      1 2 3 0))
   "Regular expressions for matching file positions in Macaulay2 output.")
 
@@ -378,18 +380,15 @@ can be executed with \\[M2-send-to-program]."
     ("^[0-9][0-9]$" nil))
   "List of filenames not to match in Macaulay2 output.")
 
-(defun M2-send-input ()
-  "Send the input to the Macaulay2 interpreter using `comint-send-input'."
-  (interactive)
-  (let ((comint-use-prompt-regexp t))
-    (comint-send-input)))
+(define-obsolete-function-alias
+  'M2-send-input 'comint-send-input "1.23")
 
 (define-obsolete-function-alias
-  'M2-send-to-program-or-jump-to-source-code 'M2-send-input "1.22")
+  'M2-send-to-program-or-jump-to-source-code 'comint-send-input "1.22")
 
 (defun M2--get-send-to-buffer ()
-  "Helper function for `M2-send-to-program` and friends.  Gets buffer for
-Macaulay2 inferior process from minibuffer or history."
+  "Helper function for `M2-send-to-program' and friends.
+Gets buffer for Macaulay2 inferior process from minibuffer or history."
   (list
    (cond (current-prefix-arg
 	  (read-from-minibuffer "buffer to send command to: " "*M2*" nil nil
@@ -397,51 +396,21 @@ Macaulay2 inferior process from minibuffer or history."
 	 (t (car M2-send-to-buffer-history)))))
 
 (defun M2--send-to-program-helper (send-to-buffer start end)
-  "Helper function for `M2-send-to-program` and friends.  Sends code between
-START and END to Macaulay2 inferior process in SEND-TO-BUFFER."
-  (or (get-buffer-window send-to-buffer 'visible)
-	 (pop-to-buffer (prog1 (current-buffer) (pop-to-buffer send-to-buffer))))
-     (select-window
-      (prog1
-	  (selected-window)
-	  (let* ((send-it t)
-		 (cmd (if (and
-			  (equal (point) (point-max))
-			  (equal (current-buffer)
-				 (with-current-buffer send-to-buffer)))
-			 (if (equal (point)
-				    (save-excursion
-				      (M2-to-end-of-prompt)
-				      (if (looking-at "[ \t]+") (goto-char (match-end 0)))
-				      (point)))
-			     (let* ((s (current-buffer))
-				    (bol (progn (beginning-of-line) (point)))
-				    (eol (progn (end-of-line) (point)))
-				    (eob (point-max))
-				    (cmd (if (equal bol eob)
-					     (concat "-- end of buffer "
-						     (if (stringp M2-demo-buffer)
-							 M2-demo-buffer
-						       (buffer-name M2-demo-buffer)))
-					   (buffer-substring bol eol))))
-			       (end-of-line)
-			       (forward-line)
-			       (set-window-point
-				(get-buffer-window (current-buffer) 'visible)
-				(point))
-			       (set-buffer s)
-			       (setq send-it nil)
-			       cmd)
-			   "")
-			(M2-blink-region start end)
-			(buffer-substring start end))))
-	    (progn
-	      (select-window (get-buffer-window (set-buffer send-to-buffer) 'visible))
-	      (goto-char (point-max))
-	      (insert cmd)
-	      (goto-char (point-max))
-	      (set-window-point (get-buffer-window send-to-buffer 'visible) (point))
-	      (if send-it (comint-send-input)))))))
+  "Helper function for `M2-send-to-program' and friends.
+Sends code between START and END to Macaulay2 inferior process in
+SEND-TO-BUFFER."
+  (unless (and (get-buffer send-to-buffer) (get-buffer-process send-to-buffer))
+    (user-error
+     "Start a Macaulay2 process first with `M-x M2' or `%s'."
+     (key-description (where-is-internal #'M2 overriding-local-map t))))
+  (display-buffer send-to-buffer '(nil (inhibit-same-window . t)))
+  (let ((cmd (buffer-substring start end)))
+    (M2-blink-region start end)
+    (with-current-buffer send-to-buffer
+      (goto-char (point-max))
+      (insert cmd)
+      (comint-send-input)
+      (set-window-point (get-buffer-window send-to-buffer 'visible) (point)))))
 
 (defun M2-send-region-to-program (send-to-buffer)
   "Send the current region to Macaulay2.  See `M2-send-to-program' for more."
@@ -460,12 +429,9 @@ START and END to Macaulay2 inferior process in SEND-TO-BUFFER."
      "Send the current line except for a possible prompt, or the region, if the
 mark is active, to Macaulay2 in its buffer, making its window visible.
 Afterwards, in the case where the mark is not active, move the cursor to
-the next line.  Alternatively, if the point is at a prompt or a blank line
-at the end of the buffer *M2*, get the next line of input from demo buffer
-set by `M2-set-demo-buffer', or if it's at the end of the buffer *M2* with a
-line of input already there, submit it.  With a prefix argument, the name of
-the buffer to which this and future uses of the command (in this buffer) should
-be sent can be entered, with history."
+the next line.  With a prefix argument, the name of the buffer to
+which this and future uses of the command (in this buffer) should be
+sent can be entered, with history."
      (interactive (M2--get-send-to-buffer))
      (if (region-active-p)
 	 (M2-send-region-to-program send-to-buffer)
@@ -499,7 +465,7 @@ for more."
 
 (defun M2-set-demo-buffer()
   "Set the variable M2-demo-buffer to the current buffer, so that later,
-`M2-send-to-program' can obtain lines from this buffer."
+`M2-get-input-from-demo-buffer' can obtain lines from this buffer."
   (interactive)
   (setq M2-demo-buffer (current-buffer)))
 
@@ -523,19 +489,41 @@ for more."
 		  (modeline . nil);; doesn't work
 		  (name . "DEMO"))))
 	      (toggle-scroll-bar 0)
-	      (set-frame-font ; use (w32-select-font) to get good font names under windows
-	       (cond ((eq window-system 'w32) "-*-Lucida Console-bold-r-*-*-19-142-*-*-c-*-*-ansi-")
-		     ((eq window-system 'x) "-adobe-courier-bold-r-normal--24-240-75-75-m-150-iso8859-1")
-		     (t "12x24"))))))
+	      (set-frame-font (font-spec :size 24.0)))))
     (modify-frame-parameters f '((left + 20) (top + 30)))
     ; (M2)
     (with-current-buffer "*M2*"
       (setq comint-scroll-show-maximum-output t))))
 
+(defun M2-get-input-from-demo-buffer ()
+  "Copy the current line from `M2-demo-buffer' to the prompt."
+  (interactive)
+  (insert (with-current-buffer M2-demo-buffer
+	    (prog1
+		(if (eobp)
+		    (concat "-- end of buffer " (buffer-name (current-buffer)))
+		  (buffer-substring
+		   (prog2 (M2-to-end-of-prompt) (point))
+		   (line-end-position)))
+	      (forward-line)))))
+
+(defun M2-send-input-or-get-input-from-demo-buffer ()
+  "Either send input to Macaulay2 or get input from the demo buffer.
+If current line is blank, then copy the current line of `M2-demo-buffer'.
+Otherwise, send the input to Macaulay2."
+  (interactive)
+  (if (save-excursion (M2-to-end-of-prompt) (looking-at-p "[[:blank:]]*$"))
+      (M2-get-input-from-demo-buffer)
+    (comint-send-input)))
+
 (defun M2-info-help (string)
-  (save-excursion
-    (when (string-match "-\\* infoHelp: \\(.*\\) \\*-" string)
-      (info-other-window (match-string 1 string)))))
+  (if (string-match "-\\* infoHelp: \\(.*\\) \\*-" string)
+      (let ((end (1+ (match-end 0))))
+	(save-excursion
+	  (with-demoted-errors "%S"
+	    (info-other-window (match-string 1 string))))
+	(substring string end))
+    string))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; M2-mode
@@ -559,17 +547,21 @@ for more."
      (interactive)
      (insert ?\;)
      (and (eolp) (M2-next-line-blank) (= 0 (M2-paren-change))
-	 (M2-newline-and-indent)))
+	 (newline nil t)))
 
 (defun M2-next-line-indent-amount ()
      (+ (current-indentation) (* (M2-paren-change) M2-indent-level)))
 
 (defun M2-this-line-indent-amount ()
+     "Determine how much to indent the current line."
      (save-excursion
 	  (beginning-of-line)
 	  (if (bobp)
 	      0
 	      (forward-line -1)
+	      ;; if the previous line is blank, then keep going
+	      (while (and (not (bobp)) (looking-at-p "[[:blank:]]*$"))
+		(forward-line -1))
 	      (M2-next-line-indent-amount))))
 
 (defun M2-in-front ()
@@ -584,37 +576,39 @@ for more."
 	  (or (eobp)
 	      (progn (forward-char) (M2-blank-line)))))
 
-(defun M2-newline-and-indent ()
-     "Start a new line and indent it properly for Macaulay2 code."
-     (interactive)
-     (newline)
-     (indent-to (M2-this-line-indent-amount)))
+(define-obsolete-function-alias
+  'M2-newline-and-indent 'newline "1.23")
 
 (defun M2-electric-right-brace()
      (interactive)
      (self-insert-command 1)
-     (and (eolp) (M2-next-line-blank) (< (M2-paren-change) 0) (M2-newline-and-indent)))
+     (and (eolp) (M2-next-line-blank) (< (M2-paren-change) 0) (newline nil t)))
+
+(defcustom M2-insert-tab-commands '(indent-for-tab-command org-cycle)
+  "Commands for which `M2-electric-tab' should insert a tab."
+  :type '(repeat function)
+  :group 'Macaulay2)
 
 (defun M2-electric-tab ()
-     (interactive)
-     (if (or (not (M2-in-front)) (M2-blank-line))
-	 (indent-to (+ (current-column) M2-indent-level))
-	 (let ((i (M2-this-line-indent-amount))
-	       (j (current-indentation)))
-	      (if (not (= i j))
-		  (progn
-		       (if (< i j)
-			    (delete-region (progn (beginning-of-line) (point))
-					   (progn (back-to-indentation) (point)))
-			    (back-to-indentation))
-		       (indent-to i))))))
+  "`indent-line-function' for Macaulay2.
+If called by command in `M2-insert-tab-commands', and if the point is either
+to right of non-whitespace characters in the same line or if the line
+is blank, then insert `M2-indent-level' spaces.  Otherwise, indent the
+line based on the depth of the parentheses in the code."
+  (interactive)
+  (indent-to
+   (prog1 (if (and (memq this-command M2-insert-tab-commands)
+		   (or (not (M2-in-front)) (M2-blank-line)))
+	      (+ (current-column) M2-indent-level)
+	    (M2-this-line-indent-amount))
+     (delete-horizontal-space))))
 
 (defvar M2-demo-buffer
   (with-current-buffer (get-buffer-create "*M2-demo-buffer*")
     (M2-mode)
     (current-buffer))
-  "The buffer from which lines are obtained by M2-send-to-program when the
-cursor is at the end of the buffer.  Set it with M2-set-demo-buffer." )
+  "The buffer from which lines are obtained by `M2-get-input-from-demo-buffer'.
+Set it with `M2-set-demo-buffer'." )
 
 ;;; "blink" evaluated region (heavily inspired by ESS)
 
